@@ -27,7 +27,6 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 #include <glibtop.h>
-#include <glibtop/loadavg.h>
 #include <glibtop/proclist.h>
 #include <glibtop/procstate.h>
 #include <glibtop/procmem.h>
@@ -54,13 +53,15 @@
 #include <libwnck/libwnck.h>
 #endif
 
-#include "procman-app.h"
+#include "application.h"
 #include "proctable.h"
 #include "prettytable.h"
 #include "util.h"
 #include "interface.h"
 #include "selinux.h"
+#include "settings-keys.h"
 #include "cgroups.h"
+#include "treeview.h"
 
 ProcInfo::UserMap ProcInfo::users;
 ProcInfo::List ProcInfo::all;
@@ -73,146 +74,65 @@ ProcInfo* ProcInfo::find(pid_t pid)
     return (it == ProcInfo::all.end() ? NULL : it->second);
 }
 
-
 static void
-cb_columns_changed(GtkTreeView *treeview, gpointer data)
+cb_save_tree_state(gpointer, gpointer data)
 {
-    ProcmanApp * const app = static_cast<ProcmanApp *>(data);
+    GsmApplication * const app = static_cast<GsmApplication *>(data);
 
-    procman_save_tree_state(app->settings,
-                            GTK_WIDGET(treeview),
-                            "proctree");
-}
-
-
-static GtkTreeViewColumn*
-my_gtk_tree_view_get_column_with_sort_column_id(GtkTreeView *treeview, int id)
-{
-    GList *columns, *it;
-    GtkTreeViewColumn *col = NULL;
-
-    columns = gtk_tree_view_get_columns(treeview);
-
-    for(it = columns; it; it = it->next)
-    {
-        if(gtk_tree_view_column_get_sort_column_id(static_cast<GtkTreeViewColumn*>(it->data)) == id)
-        {
-            col = static_cast<GtkTreeViewColumn*>(it->data);
-            break;
-        }
-    }
-
-    g_list_free(columns);
-
-    return col;
-}
-
-
-void
-proctable_set_columns_order(GtkTreeView *treeview, GSList *order)
-{
-    GtkTreeViewColumn* last = NULL;
-    GSList *it;
-
-    for(it = order; it; it = it->next)
-    {
-        int id;
-        GtkTreeViewColumn *cur;
-
-        id = GPOINTER_TO_INT(it->data);
-
-        g_assert(id >= 0 && id < NUM_COLUMNS);
-
-        cur = my_gtk_tree_view_get_column_with_sort_column_id(treeview, id);
-
-        if(cur && cur != last)
-        {
-            gtk_tree_view_move_column_after(treeview, cur, last);
-            last = cur;
-        }
-    }
-}
-
-
-
-GSList*
-proctable_get_columns_order(GtkTreeView *treeview)
-{
-    GList *columns, *col;
-    GSList *order = NULL;
-
-    columns = gtk_tree_view_get_columns(treeview);
-
-    for(col = columns; col; col = col->next)
-    {
-        int id;
-
-        id = gtk_tree_view_column_get_sort_column_id(static_cast<GtkTreeViewColumn*>(col->data));
-        order = g_slist_prepend(order, GINT_TO_POINTER(id));
-    }
-
-    g_list_free(columns);
-
-    order = g_slist_reverse(order);
-
-    return order;
-}
-
-static gboolean
-search_equal_func(GtkTreeModel *model,
-                  gint column,
-                  const gchar *key,
-                  GtkTreeIter *iter,
-                  gpointer search_data)
-{
-    char* name;
-    char* user;
-    gboolean found;
-
-    gtk_tree_model_get(model, iter,
-                       COL_NAME, &name,
-                       COL_USER, &user,
-                       -1);
-
-    found = !((name && strcasestr(name, key))
-              || (user && strcasestr(user, key)));
-
-    g_free(name);
-    g_free(user);
-
-    return found;
+    gsm_tree_view_save_state (GSM_TREE_VIEW (app->tree));
 }
 
 static void
 cb_proctree_destroying (GtkTreeView *self, gpointer data)
 {
-    g_signal_handlers_disconnect_by_func(self, (gpointer) cb_columns_changed, data);
+    g_signal_handlers_disconnect_by_func (self,
+                                          (gpointer) cb_save_tree_state,
+                                          data);
+
+    g_signal_handlers_disconnect_by_func (gtk_tree_view_get_model (self),
+                                          (gpointer) cb_save_tree_state,
+                                          data);
 }
 
 static gboolean
 cb_tree_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
+    GtkTreePath *path;
 
-    if (gdk_event_triggers_context_menu ((GdkEvent *) event)) {
-        do_popup_menu (app, event);
-        return TRUE;
+    if (!gdk_event_triggers_context_menu ((GdkEvent *) event))
+        return FALSE;
+
+    if (!gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (app->tree), event->x, event->y, &path, NULL, NULL, NULL))
+        return FALSE;
+
+    if (!gtk_tree_selection_path_is_selected (app->selection, path)) {
+        if (!(event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)))
+            gtk_tree_selection_unselect_all (app->selection);
+        gtk_tree_selection_select_path (app->selection, path);
     }
 
-    return FALSE;
+    gtk_tree_path_free (path);
+
+    gtk_menu_popup (GTK_MENU (app->popup_menu),
+                    NULL, NULL, NULL, NULL,
+                    event->button, event->time);
+    return TRUE;
 }
 
 static gboolean
 cb_tree_popup_menu (GtkWidget *widget, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
-    do_popup_menu (app, NULL);
+    gtk_menu_popup (GTK_MENU (app->popup_menu),
+                    NULL, NULL, NULL, NULL,
+                    0, gtk_get_current_event_time ());
 
     return TRUE;
 }
 
-static void
+void
 get_last_selected (GtkTreeModel *model, GtkTreePath *path,
                    GtkTreeIter *iter, gpointer data)
 {
@@ -224,20 +144,20 @@ get_last_selected (GtkTreeModel *model, GtkTreePath *path,
 void
 cb_row_selected (GtkTreeSelection *selection, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     app->selection = selection;
 
-    app->selected_process = NULL;
+    ProcInfo *selected_process = NULL;
 
     /* get the most recent selected process and determine if there are
     ** no selected processes
     */
     gtk_tree_selection_selected_foreach (app->selection, get_last_selected,
-                                         &app->selected_process);
-    if (app->selected_process) {
+                                         &selected_process);
+    if (selected_process) {
         GVariant *priority;
-        gint nice = app->selected_process->nice;
+        gint nice = selected_process->nice;
         if (nice < -7)
             priority = g_variant_new_int32 (-20);
         else if (nice < -2)
@@ -260,10 +180,10 @@ cb_row_selected (GtkTreeSelection *selection, gpointer data)
 static gint
 cb_timeout (gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
     guint new_interval;
 
-    proctable_update_all (app);
+    proctable_update (app);
 
     if (app->smooth_refresh->get(new_interval)) {
         app->timeout = g_timeout_add(new_interval,
@@ -278,7 +198,7 @@ cb_timeout (gpointer data)
 static void
 cb_refresh_icons (GtkIconTheme *theme, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     if(app->timeout) {
         g_source_remove (app->timeout);
@@ -291,12 +211,110 @@ cb_refresh_icons (GtkIconTheme *theme, gpointer data)
     cb_timeout(app);
 }
 
+static gboolean
+iter_matches_search_key (GtkTreeModel *model, GtkTreeIter *iter, const gchar *key)
+{
+    char *name;
+    char *user;
+    gboolean found;
+
+    gtk_tree_model_get (model, iter,
+                        COL_NAME, &name,
+                        COL_USER, &user,
+                        -1);
+
+    found = (name && strcasestr (name, key)) || (user && strcasestr (user, key));
+
+    g_free (name);
+    g_free (user);
+
+    return found;
+}
+
+static gboolean
+process_visibility_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+    GsmApplication * const app = static_cast<GsmApplication *>(data);
+    const gchar * search_text = app->search_entry == NULL ? "" : gtk_entry_get_text (GTK_ENTRY (app->search_entry));
+    GtkTreePath *tree_path = gtk_tree_model_get_path (model, iter);
+
+    if (strcmp (search_text, "") == 0)
+        return TRUE;
+
+	// in case we are in dependencies view, we show (and expand) rows not matching the text, but having a matching child
+    gboolean match = false;
+    if (g_settings_get_boolean (app->settings, GSM_SETTING_SHOW_DEPENDENCIES)) {
+        GtkTreeIter child;
+        if (gtk_tree_model_iter_children (model, &child, iter)) {
+            gboolean child_match = FALSE;
+            do {
+                child_match = process_visibility_func (model, &child, data);
+            } while (gtk_tree_model_iter_next (model, &child) && !child_match);
+            match = child_match;
+        }
+
+        match |= iter_matches_search_key (model, iter, search_text);
+        // TODO auto-expand items not matching the search string but having matching children
+        // complicated because of treestore nested in treemodelfilter nested in treemodelsort
+        // expand to path requires the path string in the treemodelsort, but tree_path is the path in the double nested treestore
+        //if (match && (strlen (search_text) > 0)) {
+        //    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (app->tree), tree_path);
+        //}
+
+    } else {
+        match = iter_matches_search_key (model, iter, search_text);
+    }
+        
+    gtk_tree_path_free (tree_path);
+
+    return match;
+}
+
+static void
+proctable_clear_tree (GsmApplication * const app)
+{
+    GtkTreeModel *model;
+
+    model =  gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (
+             gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT (
+             gtk_tree_view_get_model (GTK_TREE_VIEW(app->tree))))));
+
+    gtk_tree_store_clear (GTK_TREE_STORE (model));
+
+    proctable_free_table (app);
+
+    update_sensitivity(app);
+}
+                                                        
+static void
+cb_show_dependencies_changed (GSettings *settings, const gchar *key, gpointer data)
+{
+    GsmApplication *app = (GsmApplication *) data;
+
+    gtk_tree_view_set_show_expanders (GTK_TREE_VIEW (app->tree),
+                                      g_settings_get_boolean (settings, GSM_SETTING_SHOW_DEPENDENCIES));
+
+    proctable_clear_tree (app);
+    proctable_update (app);
+}
+
+static void
+cb_show_whose_processes_changed (GSettings *settings, const gchar *key, gpointer data)
+{
+    GsmApplication *app = (GsmApplication *) data;
+
+    proctable_clear_tree (app);
+    proctable_update (app);
+}
+
 GtkWidget *
-proctable_new (ProcmanApp * const app)
+proctable_new (GsmApplication * const app)
 {
     GtkWidget *proctree;
     GtkTreeStore *model;
-    GtkTreeSelection *selection;
+    GtkTreeModelFilter *model_filter;
+    GtkTreeModelSort *model_sort;
+    
     GtkTreeViewColumn *column;
     GtkCellRenderer *cell_renderer;
     const gchar *titles[] = {
@@ -331,7 +349,7 @@ proctable_new (ProcmanApp * const app)
     };
 
     gint i;
-    GSettings * settings = g_settings_get_child (app->settings, "proctree");
+    GSettings * settings = g_settings_get_child (app->settings, GSM_SETTINGS_CHILD_PROCESSES);
     model = gtk_tree_store_new (NUM_COLUMNS,
                                 G_TYPE_STRING,      /* Process Name */
                                 G_TYPE_STRING,      /* User         */
@@ -361,20 +379,22 @@ proctable_new (ProcmanApp * const app)
                                 G_TYPE_STRING       /* Sexy tooltip */
         );
 
-    proctree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+    model_filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL));
+        
+    gtk_tree_model_filter_set_visible_func(model_filter, process_visibility_func, app, NULL);
+    
+    model_sort = GTK_TREE_MODEL_SORT (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model_filter)));
+    
+    proctree = gsm_tree_view_new (settings, TRUE);
+    gtk_tree_view_set_model (GTK_TREE_VIEW (proctree), GTK_TREE_MODEL (model_sort));
+
     gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (proctree), COL_TOOLTIP);
     gtk_tree_view_set_show_expanders (GTK_TREE_VIEW (proctree),
-                                      g_settings_get_boolean (app->settings, "show-dependencies"));
-    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (proctree),
-                                         search_equal_func,
-                                         NULL,
-                                         NULL);
+                                      g_settings_get_boolean (app->settings, GSM_SETTING_SHOW_DEPENDENCIES));
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (proctree), FALSE);
     gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (proctree), TRUE);
     g_object_unref (G_OBJECT (model));
 
-    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (proctree));
-    gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
-
     column = gtk_tree_view_column_new ();
 
     cell_renderer = gtk_cell_renderer_pixbuf_new ();
@@ -391,12 +411,12 @@ proctable_new (ProcmanApp * const app)
     gtk_tree_view_column_set_title (column, _(titles[0]));
 
     gtk_tree_view_column_set_sort_column_id (column, COL_NAME);
-    bind_column_to_gsetting (settings, column);
     gtk_tree_view_column_set_resizable (column, TRUE);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_min_width (column, 1);
     gtk_tree_view_column_set_reorderable(column, TRUE);
-    gtk_tree_view_append_column (GTK_TREE_VIEW (proctree), column);
+
+    gsm_tree_view_append_and_bind_column (GSM_TREE_VIEW (proctree), column);
     gtk_tree_view_set_expander_column (GTK_TREE_VIEW (proctree), column);
 
     for (i = COL_USER; i <= COL_PRIORITY; i++) {
@@ -404,19 +424,21 @@ proctable_new (ProcmanApp * const app)
         GtkCellRenderer *cell;
 
 #ifndef HAVE_WNCK
-        if (i == COL_MEMXSERVER) {
+        if (i == COL_MEMXSERVER)
           continue;
-        }
 #endif
+
+        if (i == COL_MEMWRITABLE)
+            continue;
+
         cell = gtk_cell_renderer_text_new();
         col = gtk_tree_view_column_new();
         gtk_tree_view_column_pack_start(col, cell, TRUE);
         gtk_tree_view_column_set_title(col, _(titles[i]));
         gtk_tree_view_column_set_resizable(col, TRUE);
         gtk_tree_view_column_set_sort_column_id(col, i);
-        bind_column_to_gsetting (settings, col);
         gtk_tree_view_column_set_reorderable(col, TRUE);
-        gtk_tree_view_append_column(GTK_TREE_VIEW(proctree), col);
+        gsm_tree_view_append_and_bind_column (GSM_TREE_VIEW (proctree), col);
 
         // type
         switch (i) {
@@ -432,7 +454,6 @@ proctable_new (ProcmanApp * const app)
             case COL_MEMRES:
             case COL_MEMSHARED:
             case COL_MEM:
-            case COL_MEMWRITABLE:
                 gtk_tree_view_column_set_cell_data_func(col, cell,
                                                         &procman::size_na_cell_data_func,
                                                         GUINT_TO_POINTER(i),
@@ -479,18 +500,18 @@ proctable_new (ProcmanApp * const app)
             case COL_MEMRES:
             case COL_MEMSHARED:
             case COL_MEM:
-            case COL_MEMWRITABLE:
             case COL_CPU:
             case COL_CPU_TIME:
             case COL_START_TIME:
-                gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), i,
-                                                procman::number_compare_func, GUINT_TO_POINTER(i),
-                                                NULL);
+                gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model_sort), i,
+                                                 procman::number_compare_func,
+                                                 GUINT_TO_POINTER (i),
+                                                 NULL);
                 break;
             case COL_PRIORITY:
-                gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), i,
-                                                procman::priority_compare_func,
-                                                GUINT_TO_POINTER(COL_NICE), NULL);
+                gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model_sort), i,
+                                                 procman::priority_compare_func,
+                                                 GUINT_TO_POINTER (COL_NICE), NULL);
                 break;
             default:
                 break;
@@ -501,7 +522,6 @@ proctable_new (ProcmanApp * const app)
         {
             case COL_VMSIZE:
             case COL_MEMRES:
-            case COL_MEMWRITABLE:
             case COL_MEMSHARED:
 #ifdef HAVE_WNCK
             case COL_MEMXSERVER:
@@ -531,37 +551,31 @@ proctable_new (ProcmanApp * const app)
     app->last_vscroll_max = 0;
     app->last_vscroll_value = 0;
 
-    procman_get_tree_state (app->settings, proctree, "proctree");
-    /* Override column settings by hiding this column if it's meaningless: */
-    if (!can_show_security_context_column ()) {
-        GtkTreeViewColumn *column;
-        column = my_gtk_tree_view_get_column_with_sort_column_id (GTK_TREE_VIEW (proctree), COL_SECURITYCONTEXT);
-        gtk_tree_view_column_set_visible (column, FALSE);
+    if (!cgroups_enabled ())
+        gsm_tree_view_add_excluded_column (GSM_TREE_VIEW (proctree), COL_CGROUP);
+
+#ifdef HAVE_SYSTEMD
+    if (!LOGIND_RUNNING ())
+#endif
+    {
+        gsm_tree_view_add_excluded_column (GSM_TREE_VIEW (proctree), COL_UNIT);
+        gsm_tree_view_add_excluded_column (GSM_TREE_VIEW (proctree), COL_SESSION);
+        gsm_tree_view_add_excluded_column (GSM_TREE_VIEW (proctree), COL_SEAT);
+        gsm_tree_view_add_excluded_column (GSM_TREE_VIEW (proctree), COL_OWNER);
     }
 
-    if (!cgroups_enabled()) {
-        GtkTreeViewColumn *column;
+    if (!can_show_security_context_column ())
+        gsm_tree_view_add_excluded_column (GSM_TREE_VIEW (proctree), COL_SECURITYCONTEXT);
 
-        column = my_gtk_tree_view_get_column_with_sort_column_id(GTK_TREE_VIEW(proctree), COL_CGROUP);
-        gtk_tree_view_column_set_visible(column, FALSE);
-    }
+    gsm_tree_view_load_state (GSM_TREE_VIEW (proctree));
 
     GtkIconTheme* theme = gtk_icon_theme_get_default();
     g_signal_connect(G_OBJECT (theme), "changed", G_CALLBACK (cb_refresh_icons), app);
+
+    app->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (proctree));
+    gtk_tree_selection_set_mode (app->selection, GTK_SELECTION_MULTIPLE);
     
-#ifdef HAVE_SYSTEMD
-    if (!LOGIND_RUNNING())
-#endif
-    {
-        GtkTreeViewColumn *column;
-
-        for (i = COL_UNIT; i <= COL_OWNER; i++) {
-            column = my_gtk_tree_view_get_column_with_sort_column_id(GTK_TREE_VIEW(proctree), i);
-            gtk_tree_view_column_set_visible(column, FALSE);
-        }
-    }
-
-    g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (proctree))),
+    g_signal_connect (G_OBJECT (app->selection),
                       "changed",
                       G_CALLBACK (cb_row_selected), app);
     g_signal_connect (G_OBJECT (proctree), "popup_menu",
@@ -569,17 +583,27 @@ proctable_new (ProcmanApp * const app)
     g_signal_connect (G_OBJECT (proctree), "button_press_event",
                       G_CALLBACK (cb_tree_button_pressed), app);
 
-    g_signal_connect (G_OBJECT(proctree), "destroy",
-                      G_CALLBACK(cb_proctree_destroying),
+    g_signal_connect (G_OBJECT (proctree), "destroy",
+                      G_CALLBACK (cb_proctree_destroying),
                       app);
 
-    g_signal_connect (G_OBJECT(proctree), "columns-changed",
-                      G_CALLBACK(cb_columns_changed), app);
+    g_signal_connect (G_OBJECT (proctree), "columns-changed",
+                      G_CALLBACK (cb_save_tree_state), app);
+
+    g_signal_connect (G_OBJECT (model_sort), "sort-column-changed",
+                      G_CALLBACK (cb_save_tree_state), app);
+
+    g_signal_connect (app->settings, "changed::" GSM_SETTING_SHOW_DEPENDENCIES,
+                      G_CALLBACK (cb_show_dependencies_changed), app);
+
+    g_signal_connect (app->settings, "changed::" GSM_SETTING_SHOW_WHOSE_PROCESSES,
+                      G_CALLBACK (cb_show_whose_processes_changed), app);
+
+    gtk_widget_show (proctree);
 
     return proctree;
 }
 
-
 ProcInfo::~ProcInfo()
 {
     g_free(this->name);
@@ -595,7 +619,6 @@ ProcInfo::~ProcInfo()
     free(this->seat);
 }
 
-
 static void
 get_process_name (ProcInfo *info,
                   const gchar *cmd, const GStrv args)
@@ -655,7 +678,8 @@ ProcInfo::set_user(guint uid)
     this->user = lookup_user(uid);
 }
 
-static void get_process_memory_writable(ProcInfo *info)
+void
+get_process_memory_writable (ProcInfo *info)
 {
     glibtop_proc_map buf;
     glibtop_map_entry *maps;
@@ -699,22 +723,19 @@ get_process_memory_info(ProcInfo *info)
     info->memres    = procmem.resident;
     info->memshared = procmem.share;
 
-    get_process_memory_writable(info);
-
-    // fake the smart memory column if writable is not available
-    info->mem = info->memwritable ? info->memwritable : info->memres;
+    info->mem = info->memres - info->memshared;
 #ifdef HAVE_WNCK
     info->mem += info->memxserver;
 #endif
 }
 
-
-
 static void
 update_info_mutable_cols(ProcInfo *info)
 {
     GtkTreeModel *model;
-    model = gtk_tree_view_get_model(GTK_TREE_VIEW(ProcmanApp::get()->tree));
+    model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (
+            gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(
+            gtk_tree_view_get_model (GTK_TREE_VIEW(GsmApplication::get()->tree))))));
 
     using procman::tree_store_update;
 
@@ -722,7 +743,6 @@ update_info_mutable_cols(ProcInfo *info)
     tree_store_update(model, &info->node, COL_USER, info->user.c_str());
     tree_store_update(model, &info->node, COL_VMSIZE, info->vmsize);
     tree_store_update(model, &info->node, COL_MEMRES, info->memres);
-    tree_store_update(model, &info->node, COL_MEMWRITABLE, info->memwritable);
     tree_store_update(model, &info->node, COL_MEMSHARED, info->memshared);
 #ifdef HAVE_WNCK
     tree_store_update(model, &info->node, COL_MEMXSERVER, info->memxserver);
@@ -740,16 +760,17 @@ update_info_mutable_cols(ProcInfo *info)
     tree_store_update(model, &info->node, COL_OWNER, info->owner.c_str());
 }
 
-
-
 static void
-insert_info_to_tree (ProcInfo *info, ProcmanApp *app, bool forced = false)
+insert_info_to_tree (ProcInfo *info, GsmApplication *app, bool forced = false)
 {
     GtkTreeModel *model;
-
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW (app->tree));
-
-    if (g_settings_get_boolean (app->settings, "show-dependencies")) {
+    GtkTreeModel *filtered;
+    GtkTreeModel *sorted;
+    sorted = gtk_tree_view_get_model (GTK_TREE_VIEW(app->tree));
+    filtered = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(sorted));
+    model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filtered));
+    
+    if (g_settings_get_boolean (app->settings, GSM_SETTING_SHOW_DEPENDENCIES)) {
 
         ProcInfo *parent = 0;
 
@@ -759,15 +780,24 @@ insert_info_to_tree (ProcInfo *info, ProcmanApp *app, bool forced = false)
         if (parent) {
             GtkTreePath *parent_node = gtk_tree_model_get_path(model, &parent->node);
             gtk_tree_store_insert(GTK_TREE_STORE(model), &info->node, &parent->node, 0);
-
-            if (!gtk_tree_view_row_expanded(GTK_TREE_VIEW(app->tree), parent_node)
+            
+            GtkTreePath *filtered_parent = gtk_tree_model_filter_convert_child_path_to_path (GTK_TREE_MODEL_FILTER (filtered), parent_node);
+            if (filtered_parent != NULL) {
+                GtkTreePath *sorted_parent = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (sorted), filtered_parent);
+            
+                if (sorted_parent != NULL) {
+                    if (!gtk_tree_view_row_expanded(GTK_TREE_VIEW(app->tree), sorted_parent)
 #ifdef __linux__
-                // on linuxes we don't want to expand kthreadd by default (always has pid 2)
-                && (parent->pid != 2)
+                        // on linuxes we don't want to expand kthreadd by default (always has pid 2)
+                        && (parent->pid != 2)
 #endif
-            )
-                gtk_tree_view_expand_row(GTK_TREE_VIEW(app->tree), parent_node, FALSE);
-            gtk_tree_path_free(parent_node);
+                    )
+                        gtk_tree_view_expand_row(GTK_TREE_VIEW(app->tree), sorted_parent, FALSE);
+                    gtk_tree_path_free (sorted_parent);
+                }
+                gtk_tree_path_free (filtered_parent);
+            }
+            gtk_tree_path_free (parent_node);
         } else
             gtk_tree_store_insert(GTK_TREE_STORE(model), &info->node, NULL, 0);
     }
@@ -788,13 +818,12 @@ insert_info_to_tree (ProcInfo *info, ProcmanApp *app, bool forced = false)
     procman_debug("inserted %d%s", info->pid, (forced ? " (forced)" : ""));
 }
 
-
 /* Removing a node with children - make sure the children are queued
 ** to be readded.
 */
 template<typename List>
 static void
-remove_info_from_tree (ProcmanApp *app, GtkTreeModel *model,
+remove_info_from_tree (GsmApplication *app, GtkTreeModel *model,
                        ProcInfo *current, List &orphans, unsigned lvl = 0)
 {
     GtkTreeIter child_node;
@@ -819,9 +848,6 @@ remove_info_from_tree (ProcmanApp *app, GtkTreeModel *model,
 
     g_assert(not gtk_tree_model_iter_has_child(model, &current->node));
 
-    if (app->selected_process == current)
-        app->selected_process = NULL;
-
     orphans.push_back(current);
     gtk_tree_store_remove(GTK_TREE_STORE(model), &current->node);
     procman::poison(current->node, 0x69);
@@ -858,7 +884,7 @@ get_process_systemd_info(ProcInfo *info)
 }
 
 static void
-update_info (ProcmanApp *app, ProcInfo *info)
+update_info (GsmApplication *app, ProcInfo *info)
 {
     glibtop_proc_state procstate;
     glibtop_proc_uid procuid;
@@ -899,7 +925,6 @@ update_info (ProcmanApp *app, ProcInfo *info)
     get_process_systemd_info(info);
 }
 
-
 ProcInfo::ProcInfo(pid_t pid)
     : tooltip(NULL),
       name(NULL),
@@ -949,12 +974,14 @@ ProcInfo::ProcInfo(pid_t pid)
 }
 
 static void
-refresh_list (ProcmanApp *app, const pid_t* pid_list, const guint n)
+refresh_list (GsmApplication *app, const pid_t* pid_list, const guint n)
 {
     typedef std::list<ProcInfo*> ProcList;
     ProcList addition;
 
-    GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (app->tree));
+    GtkTreeModel    *model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (
+                             gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT (
+                             gtk_tree_view_get_model (GTK_TREE_VIEW(app->tree))))));
     guint i;
 
     // Add or update processes in the process list
@@ -999,7 +1026,7 @@ refresh_list (ProcmanApp *app, const pid_t* pid_list, const guint n)
     // pid_list == ProcInfo::all + addition
 
 
-    if (g_settings_get_boolean (app->settings, "show-dependencies")) {
+    if (g_settings_get_boolean (app->settings, GSM_SETTING_SHOW_DEPENDENCIES)) {
 
         // insert process in the tree. walk through the addition list
         // (new process + process that have a new parent). This loop
@@ -1032,7 +1059,7 @@ refresh_list (ProcmanApp *app, const pid_t* pid_list, const guint n)
                 // FIXME: this is broken if the unreachable parent becomes active
                 // i.e. it gets active or changes ower
                 // so we just clear the tree on __each__ update
-                // see proctable_update_list (ProcData * const procdata)
+                // see proctable_update (ProcData * const procdata)
 
 
                 if ((*it)->ppid == 0 or in_tree.find((*it)->ppid) != in_tree.end()) {
@@ -1067,9 +1094,8 @@ refresh_list (ProcmanApp *app, const pid_t* pid_list, const guint n)
         update_info_mutable_cols(it->second);
 }
 
-
 void
-proctable_update_list (ProcmanApp *app)
+proctable_update (GsmApplication *app)
 {
     pid_t* pid_list;
     glibtop_proclist proclist;
@@ -1077,7 +1103,7 @@ proctable_update_list (ProcmanApp *app)
     int which = 0;
     int arg = 0;
 
-    const char* whose_processes = g_settings_get_string (app->settings, "show-whose-processes");
+    char *whose_processes = g_settings_get_string (app->settings, GSM_SETTING_SHOW_WHOSE_PROCESSES);
     if (strcmp (whose_processes, "all") == 0) {
         which = GLIBTOP_KERN_PROC_ALL;
         arg = 0;
@@ -1088,6 +1114,7 @@ proctable_update_list (ProcmanApp *app)
       which = GLIBTOP_KERN_PROC_UID;
       arg = getuid ();
     }
+    g_free (whose_processes);
 
     pid_list = glibtop_get_proclist (&proclist, which, arg);
 
@@ -1097,32 +1124,31 @@ proctable_update_list (ProcmanApp *app)
     app->cpu_total_time = MAX(cpu.total - app->cpu_total_time_last, 1);
     app->cpu_total_time_last = cpu.total;
     
-    GtkAdjustment* vadjustment = GTK_ADJUSTMENT(gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(app->tree)));
     refresh_list (app, pid_list, proclist.number);
-    
+
     // juggling with tree scroll position to fix https://bugzilla.gnome.org/show_bug.cgi?id=92724
     GtkTreePath* current_top;
     if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(app->tree), 0,0, &current_top, NULL, NULL, NULL)) {
+        GtkAdjustment *vadjustment = GTK_ADJUSTMENT (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (app->tree)));
         gdouble current_max = gtk_adjustment_get_upper(vadjustment);
         gdouble current_value = gtk_adjustment_get_value(vadjustment);
+
         if (app->top_of_tree) {
             // if the visible cell from the top of the tree is still the same, as last time 
-            if (gtk_tree_path_compare(app->top_of_tree, current_top) == 0) 
-            {
+            if (gtk_tree_path_compare (app->top_of_tree, current_top) == 0) {
                 //but something from the scroll parameters has changed compared to the last values
-                if (app->last_vscroll_value==0 && current_value!=app->last_vscroll_value) 
-                {
+                if (app->last_vscroll_value == 0 && current_value != 0) {
                     // the tree was scrolled to top, and something has been added above the current top row
                     gtk_tree_view_scroll_to_point(GTK_TREE_VIEW(app->tree), -1, 0);
-                }
-                else if (current_max > app->last_vscroll_max && current_value == app->last_vscroll_value)
-                {
+                } else if (current_max > app->last_vscroll_max && app->last_vscroll_max == app->last_vscroll_value) {
                     // the tree was scrolled to bottom, something has been added below the current bottom row
                     gtk_tree_view_scroll_to_point(GTK_TREE_VIEW(app->tree), -1, current_max);
                 }
             }
+
             gtk_tree_path_free(app->top_of_tree);
         }
+
         app->top_of_tree = current_top;
         app->last_vscroll_value = current_value;
         app->last_vscroll_max = current_max;
@@ -1133,37 +1159,8 @@ proctable_update_list (ProcmanApp *app)
     /* proclist.number == g_list_length(procdata->info) == g_hash_table_size(procdata->pids) */
 }
 
-
 void
-proctable_update_all (ProcmanApp * const app)
-{
-    char* string;
-
-    string = make_loadavg_string();
-    gtk_label_set_text (GTK_LABEL(app->loadavg), string);
-    g_free (string);
-
-    proctable_update_list (app);
-}
-
-
-void
-proctable_clear_tree (ProcmanApp * const app)
-{
-    GtkTreeModel *model;
-
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW (app->tree));
-
-    gtk_tree_store_clear (GTK_TREE_STORE (model));
-
-    proctable_free_table (app);
-
-    update_sensitivity(app);
-}
-
-
-void
-proctable_free_table (ProcmanApp * const app)
+proctable_free_table (GsmApplication * const app)
 {
     for (ProcInfo::Iterator it(ProcInfo::begin()); it != ProcInfo::end(); ++it)
         delete it->second;
@@ -1171,39 +1168,22 @@ proctable_free_table (ProcmanApp * const app)
     ProcInfo::all.clear();
 }
 
-
-
-char*
-make_loadavg_string(void)
-{
-    glibtop_loadavg buf;
-
-    glibtop_get_loadavg(&buf);
-
-    return g_strdup_printf(
-        _("Load averages for the last 1, 5, 15 minutes: "
-          "%0.2f, %0.2f, %0.2f"),
-        buf.loadavg[0],
-        buf.loadavg[1],
-        buf.loadavg[2]);
-}
-
-
-
 void
 ProcInfo::set_icon(Glib::RefPtr<Gdk::Pixbuf> icon)
 {
     this->pixbuf = icon;
 
     GtkTreeModel *model;
-    model = gtk_tree_view_get_model(GTK_TREE_VIEW(ProcmanApp::get()->tree));
+    model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (
+            gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(
+            gtk_tree_view_get_model (GTK_TREE_VIEW(GsmApplication::get()->tree))))));
     gtk_tree_store_set(GTK_TREE_STORE(model), &this->node,
                        COL_PIXBUF, (this->pixbuf ? this->pixbuf->gobj() : NULL),
                        -1);
 }
 
 void
-proctable_freeze (ProcmanApp *app)
+proctable_freeze (GsmApplication *app)
 {
     if (app->timeout) {
       g_source_remove (app->timeout);
@@ -1212,15 +1192,18 @@ proctable_freeze (ProcmanApp *app)
 }
 
 void
-proctable_thaw (ProcmanApp *app)
+proctable_thaw (GsmApplication *app)
 {
+    if (app->timeout)
+        return;
+
     app->timeout = g_timeout_add (app->config.update_interval,
                                   cb_timeout,
                                   app);
 }
 
 void
-proctable_reset_timeout (ProcmanApp *app)
+proctable_reset_timeout (GsmApplication *app)
 {
     proctable_freeze (app);
     proctable_thaw (app);
